package com.tuoheng.admin.service.third.airport;

import com.alibaba.fastjson.JSONObject;
import com.tuoheng.admin.constant.SystemConstant;
import com.tuoheng.admin.entity.dto.AirportDetailDto;
import com.tuoheng.admin.entity.dto.MountEquipment;
import com.tuoheng.common.core.config.common.CommonConfig;
import com.tuoheng.common.core.utils.HttpUtils;
import com.tuoheng.common.core.utils.JacksonUtil;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * @Author xiaoying
 * @Date 2023/4/14 16:05
 */
@Service
@Slf4j
public class GetAirportDetailService {

    public JsonResult getAirportDetail(Integer airportId) {
        String url = CommonConfig.airportURL + SystemConstant.API_AIRPORT_STATUS;
        String param = "airportId=" + airportId;
        String result = HttpUtils.sendGet(url, param);
        JsonResult jsonResult = JacksonUtil.json2pojo(result, JsonResult.class);
        if (0 != jsonResult.getCode()) {
            log.info("调用机场平台，获取天气信息失败，{}", jsonResult);
            return JsonResult.error(jsonResult.getMsg());
        }
        AirportDetailDto vo = new AirportDetailDto();
        JSONObject dataObject = (JSONObject) JSONObject.toJSON(jsonResult.getData());

//        log.info("响应信息:{}", dataObject.toJSONString());
        JSONObject wthJson = dataObject.getJSONObject("WTH");
        if (Objects.nonNull(wthJson)) {
//            log.info("参数信息:{}", wthJson.toJSONString());
            JSONObject parmJson = wthJson.getJSONObject("parmNew");
//            log.info("参数信息parmJson:{}", parmJson.toJSONString());
            vo.setWspd(parmJson.getString("WSPD") + "m/s")
                    .setWdir(parmJson.getString("WDIRNAME"))
                    .setHpa(parmJson.getString("Hpa") + "Mpa")
                    .setHum(parmJson.getString("Hum") + "%rh")
                    .setRainfull(parmJson.getString("Rainfull") + "mm")
                    .setTmp(parmJson.getString("Tmp") + "℃");
        }
        JSONObject tahJson = dataObject.getJSONObject("TAH");
        if (Objects.nonNull(tahJson)) {
            JSONObject parmJson = tahJson.getJSONObject("parmNew");
            vo.setHum(parmJson.getString("Hum") + "%rh")
                    .setTmp(parmJson.getString("Tmp") + "℃");

        }
        JSONObject mountJson = dataObject.getJSONObject("mount");
        if (Objects.nonNull(mountJson)) {
            vo.setMountName(mountJson.getString("cameraName") + "、" +
                    mountJson.getString("megaphoneName") + "、" +
                    mountJson.getString("searchlightName")
            );
        }
        if (Objects.nonNull(mountJson)) {
            String strJson = mountJson.getString("mountEquipmentList");
//            log.info("参数信息，strJson={}", strJson);
            if (StringUtils.isNotEmpty(strJson)) {
                String mountEquipmentList = strJson;
                if (StringUtils.isNotEmpty(mountEquipmentList)) {
                    List<MountEquipment> mountEquipments = JSONObject.parseArray(mountEquipmentList, MountEquipment.class);
                    vo.setMountEquipmentList(mountEquipments);
                }

            }
        }
        vo.setStatus(dataObject.getString("status"));
        return JsonResult.success(vo);
    }
}
